import { afterEvents, checkBlockPattern, Vector } from "./libraries/utils";
import Materials from "./Materials";
const GolemPatterns = [
    /**
     * ###
     *  #
     */
    [
        { offset: { x: 0, y: -1, z: 0 } },
        { offset: { x: 1, y: -1, z: 0 } },
        { offset: { x: -1, y: -1, z: 0 } },
        { offset: { x: 0, y: -2, z: 0 } },
    ],
    [
        { offset: { x: 0, y: -1, z: 0 } },
        { offset: { x: 0, y: -1, z: 1 } },
        { offset: { x: 0, y: -1, z: -1 } },
        { offset: { x: 0, y: -2, z: 0 } },
    ],
    /**
     *  #
     * ###
     */
    [
        { offset: { x: 0, y: 1, z: 0 } },
        { offset: { x: 1, y: 1, z: 0 } },
        { offset: { x: -1, y: 1, z: 0 } },
        { offset: { x: 0, y: 2, z: 0 } },
    ],
    [
        { offset: { x: 0, y: 1, z: 0 } },
        { offset: { x: 0, y: 1, z: 1 } },
        { offset: { x: 0, y: 1, z: -1 } },
        { offset: { x: 0, y: 2, z: 0 } },
    ],
    /**
     * #
     * ##
     * #
     */
    [
        { offset: { x: 1, y: 0, z: 0 } },
        { offset: { x: 2, y: 0, z: 0 } },
        { offset: { x: 1, y: 1, z: 0 } },
        { offset: { x: 1, y: -1, z: 0 } },
    ],
    [
        { offset: { x: 1, y: 0, z: 0 } },
        { offset: { x: 2, y: 0, z: 0 } },
        { offset: { x: 1, y: 0, z: 1 } },
        { offset: { x: 1, y: 0, z: -1 } },
    ],
    /**
     *  #
     * ##
     *  #
     */
    [
        { offset: { x: -1, y: 0, z: 0 } },
        { offset: { x: -2, y: 0, z: 0 } },
        { offset: { x: -1, y: 1, z: 0 } },
        { offset: { x: -1, y: -1, z: 0 } },
    ],
    [
        { offset: { x: -1, y: 0, z: 0 } },
        { offset: { x: -2, y: 0, z: 0 } },
        { offset: { x: -1, y: 0, z: 1 } },
        { offset: { x: -1, y: 0, z: -1 } },
    ],
    /////
    [
        { offset: { x: 0, y: 0, z: 1 } },
        { offset: { x: 0, y: 0, z: 2 } },
        { offset: { x: 0, y: 1, z: 1 } },
        { offset: { x: 0, y: -1, z: 1 } },
    ],
    [
        { offset: { x: 0, y: 0, z: 1 } },
        { offset: { x: 0, y: 0, z: 2 } },
        { offset: { x: 1, y: 0, z: 1 } },
        { offset: { x: -1, y: 0, z: 1 } },
    ],
    /////
    [
        { offset: { x: 0, y: 0, z: -1 } },
        { offset: { x: 0, y: 0, z: -2 } },
        { offset: { x: 0, y: 1, z: -1 } },
        { offset: { x: 0, y: -1, z: -1 } },
    ],
    [
        { offset: { x: 0, y: 0, z: -1 } },
        { offset: { x: 0, y: 0, z: -2 } },
        { offset: { x: 1, y: 0, z: -1 } },
        { offset: { x: -1, y: 0, z: -1 } },
    ],
];
afterEvents.playerPlaceBlock.subscribe(({ player, block, dimension }) => {
    if (block.typeId !== 'minecraft:carved_pumpkin')
        return;
    const origin = Vector.subtract(block.center(), { y: 2.5 });
    for (const pattern of GolemPatterns) {
        const shape = checkBlockPattern(block, pattern);
        if (!shape)
            continue;
        const material = shape[0];
        if (material.isAir || material.isLiquid)
            continue;
        const golemType = Materials[material.typeId];
        if (!golemType)
            continue;
        block.setType('minecraft:air');
        for (const block of shape) {
            block?.setType('minecraft:air');
        }
        dimension.playSound('dig.wood', origin);
        const groundY = shape.reduce((a, b) => a < b.y ? a : b.y, block.location.y);
        const ground = new Vector(origin.x, groundY, origin.z);
        const golem = dimension.spawnEntity(golemType, ground);
        golem.setRotation(Vector.multiply({ ...player.getRotation(), z: 0 }, -1));
        const tameable = golem.getComponent('tameable');
        tameable.tame(player);
        break;
    }
});
// function detectTShape(origin: Vector3, dimension: Dimension, blockType: string) {
//     const directions = [
//         { x: -1, y: 0, z: 0 }, // West
//         { x: 1, y: 0, z: 0 },  // East
//         { x: 0, y: 0, z: -1 },  // North
//         { x: 0, y: 0, z: 1 },  // South
//     ];
//     for (let dir of directions) {
//         const shape = checkTShapeInDirection(origin, dimension, blockType, dir);
//         if (shape) return shape;
//     }
//     return false;
// }
// function checkTShapeInDirection(origin: Vector3, dimension: Dimension, blockType: string, dir: Vector3) {
//     const vertical = [
//         dimension.getBlock(new Vector(origin.x, origin.y, origin.z)),
//         dimension.getBlock(new Vector(origin.x, origin.y + 1, origin.z))
//     ];
//     if (vertical.some(b => b?.typeId !== blockType)) return false;
//     const horizontal = [
//         dimension.getBlock(new Vector(origin.x + dir.x, origin.y + 1, origin.z + dir.z)),
//         dimension.getBlock(new Vector(origin.x, origin.y + 1, origin.z)),
//         dimension.getBlock(new Vector(origin.x - dir.x, origin.y + 1, origin.z - dir.z)),
//     ];
//     if (horizontal.every(b => b?.typeId === blockType)) return vertical.concat(horizontal);
//     let horizontalXZ = [
//         dimension.getBlock(new Vector(origin.x + dir.x, origin.y, origin.z + dir.z)),
//         dimension.getBlock(new Vector(origin.x, origin.y, origin.z)),
//         dimension.getBlock(new Vector(origin.x - dir.x, origin.y, origin.z - dir.z)),
//     ];
//     if (horizontalXZ.every(b => b?.typeId === blockType)) return vertical.concat(horizontalXZ);
// }
